#ifndef CONFIG_H_
#define CONFIG_H_

/* #undef CPPUTEST_MEM_LEAK_DETECTION_DISABLED */
#define CPPUTEST_USE_LONG_LONG

/* #undef CPPUTEST_STD_C_LIB_DISABLED */
/* #undef CPPUTEST_STD_CPP_LIB_DISABLED */

/* #undef INCLUDE_GTEST_TESTS */

#endif
