/*
 * Copyright (c) 2007, Michael Feathers, James Grenning and Bas Vodde
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the <organization> nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE EARLIER MENTIONED AUTHORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef D_MemoryReporterPlugin_h
#define D_MemoryReporterPlugin_h

#include "CppUTest/TestPlugin.h"
#include "CppUTestExt/MemoryReportAllocator.h"

class MemoryReportFormatter;

class MemoryReporterPlugin : public TestPlugin
{
    MemoryReportFormatter* formatter_;

    MemoryReportAllocator mallocAllocator;
    MemoryReportAllocator newAllocator;
    MemoryReportAllocator newArrayAllocator;

    SimpleString currentTestGroup_;
public:
    MemoryReporterPlugin();
    virtual ~MemoryReporterPlugin() _destructor_override;

    virtual void preTestAction(UtestShell & test, TestResult & result) _override;
    virtual void postTestAction(UtestShell & test, TestResult & result) _override;
    virtual bool parseArguments(int, const char *const *, int) _override;

    MemoryReportAllocator* getMallocAllocator();
    MemoryReportAllocator* getNewAllocator();
    MemoryReportAllocator* getNewArrayAllocator();
protected:
    virtual MemoryReportFormatter* createMemoryFormatter(const SimpleString& type);

private:
    void destroyMemoryFormatter(MemoryReportFormatter* formatter);

    void setGlobalMemoryReportAllocators();
    void removeGlobalMemoryReportAllocators();

    void initializeAllocator(MemoryReportAllocator* allocator, TestResult & result);
};

#endif
